// Straight from django docs
function getCookie(name) {
    var cookieValue = null;
    if (document.cookie && document.cookie !== '') {
        var cookies = document.cookie.split(';');
        for (var i = 0; i < cookies.length; i++) {
            var cookie = cookies[i].trim();
            // Does this cookie string begin with the name we want?
            if (cookie.substring(0, name.length + 1) === (name + '=')) {
                cookieValue = decodeURIComponent(cookie.substring(name.length + 1));
                break;
            }
        }
    }
    return cookieValue;
}

const getCreateCredentialArgsFromSSO = async () => {
    var csrftoken = getCookie('csrftoken');
    let headers = new Headers();
    headers.append('X-CSRFToken', csrftoken);
    let response = await fetch("/webauthn-register", {method: "POST", headers: headers});
    mengor = await response.json();
    return mengor;
}

const postAttestationToSSO = async(attestation_obj, old_form) => {
    var csrftoken = getCookie('csrftoken');
    let headers = new Headers();
    headers.append('X-CSRFToken', csrftoken);
    const formData = new FormData(old_form);
    Object.entries(attestation_obj).forEach(([key, value]) => {
        formData.set(key, value);
    });

    let response = await fetch(
        "/webauthn-register",
        {
            method: "POST",
            body: formData,
            headers: headers
        });
    return await response.json();
}


// See https://github.com/duo-labs/py_webauthn
// See https://developer.mozilla.org/en-US/docs/Web/API/Web_Authentication_API

const didClickRegister = async (e) => {
    e.preventDefault();
    let createCredentialArgsSSO;
    try {
        createCredentialArgsSSO = await getCreateCredentialArgsFromSSO();
    } catch (err) {
        return console.error("Error getting credential args from SSO: ", createCredentialArgsSSO)
    }
    if ("error" in createCredentialArgsSSO){
        return console.error("Error!", createCredentialArgsSSO);
    }
    // Mangle creds - maybe would be nice to copy the source and have a method that does this
    // and returns a copy with the changes (immutability yadda yadda)
    createCredentialArgsSSO.challenge = Uint8Array.from(atob(createCredentialArgsSSO.challenge), c => c.charCodeAt(0));
    createCredentialArgsSSO.user.id = Uint8Array.from(atob(createCredentialArgsSSO.user.id), c => c.charCodeAt(0));

    console.log(createCredentialArgsSSO);

    // Get the authenticator to sign the challenge and give us a set of credentials
    // including a public key and identifier
    try {
        credential = await navigator.credentials.create({
            publicKey: createCredentialArgsSSO
        }
        );
    } catch (err) {
        return console.error("Error creating credential:", err);
    }
    // log the credential
    console.log(credential);

    // Prepare this for the server to verify and store
    const att_for_server = {
        id: credential.id,
        type: credential.type,
        rawId: btoa(String.fromCharCode.apply(null, new Uint8Array(credential.rawId))),
        attObj: btoa(String.fromCharCode.apply(null, new Uint8Array(credential.response.attestationObject))),
        clientData: btoa(String.fromCharCode.apply(null, new Uint8Array(credential.response.clientDataJSON))),
        registrationClientExtensions: JSON.stringify(credential.getClientExtensionResults())
    }
    // marshaled version
    console.log(att_for_server)

    const oldform = document.forms[0];

    let postresponse;
    try {
        postresponse = await postAttestationToSSO(att_for_server, oldform);
        console.log(postresponse);
    }
    catch (err){
        return console.error("Error posting credential args to SSO: ", postresponse, err);
    }
    if ("error" in postresponse){
        return console.error("Error!", postresponse);
    }

    // If success, here reload or redirect
    window.location = "/device-list";

}

document.addEventListener("DOMContentLoaded", e => {
    document.querySelector('#register').addEventListener('click', didClickRegister);
});

// Exports mainly for testing
exports.getCookie= getCookie; 
